@@ -166,7 +166,7 @@ ngx_hash_init(ngx_hash_init_t *hinit, ngx_hash_key_t *names, ngx_uint_t nelts)
 
     bucket_size = hinit->bucket_size - sizeof(void *);
 
-    start = nelts / (bucket_size / (2 * sizeof(void *)) - 1);
+    start = nelts / (bucket_size / (2 * sizeof(void *)));
     start = start ? start : 1;
 
     if (hinit->max_size > 10000 && hinit->max_size / nelts < 100) {
