@@ -1135,7 +1135,7 @@ ngx_http_fastcgi_process_header(ngx_http_request_t *r)
 
                 } else {
                     for (i = 0; i < h->key.len; i++) {
-                        h->lowcase_key[i] = ngx_tolower(h->lowcase_key[i]);
+                        h->lowcase_key[i] = ngx_tolower(h->key.data[i]);
                     }
                 }
 
@@ -1640,7 +1640,6 @@ ngx_http_fastcgi_create_loc_conf(ngx_conf_t *cf)
      *     conf->upstream.hide_headers_hash = { NULL, 0 };
      *     conf->upstream.hide_headers = NULL;
      *     conf->upstream.pass_headers = NULL;
-     *     conf->upstream.catch_stderr = NULL;
      *     conf->upstream.schema = { 0, NULL };
      *     conf->upstream.uri = { 0, NULL };
      *     conf->upstream.location = NULL;
@@ -1675,6 +1674,8 @@ ngx_http_fastcgi_create_loc_conf(ngx_conf_t *cf)
     /* "fastcgi_cyclic_temp_file" is disabled */
     conf->upstream.cyclic_temp_file = 0;
 
+    conf->catch_stderr = NGX_CONF_UNSET_PTR;
+
     return conf;
 }
 
