@@ -167,7 +167,7 @@ ngx_http_map_variable(ngx_http_request_t *r, ngx_http_variable_value_t *v,
     }
 
     ngx_log_debug2(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
-                   "http map: \"%V\" \"%V\"", vv, v);
+                   "http map: \"%v\" \"%v\"", vv, v);
 
     return NGX_OK;
 }
